
exports.up = function (knex, Promise) {
    return knex.schema.createTable('answers', function (table) {
        table.inherits('messages');
        /*
        we require that a GraphQL Answer's author field be the same as its recipient field.
        since recipient is already known (defined in the 'messages' parent-table),
        then we simply don't store answers' authors in database.
        (another way would be to add equality constraints, but uselessly complex.)
        table.string('author', 128).notNullable()
            .references('gid').inTable('groups')
            .onDelete('CASCADE');
        */
        table.integer('for_question').notNullable()
            .references('mid').inTable('messages' /*'questions'*/)
            .onDelete('CASCADE'); //if question is deleted, also delete the answer
        /*
        ^ as stated in https://www.postgresql.org/docs/10/static/ddl-inherit.html#DDL-INHERIT-CAVEATS,
        "A serious limitation of the inheritance feature is that indexes (including unique constraints) and foreign key constraints only apply to single tables, not to their inheritance children."
        it is not possible (with PostgresQL v10) to use both inheritance and foreign keys at once.
        until it is, or a workaround is found, we simply don't maintain child-level referential integrity
        and we hope all the rest will work, e.g. simple_group_gids will indeed refer to simple groups, and idem for meta_group_gids.
        TODO: maybe, check if PostgresQL's latest version enables this feature?
        */

    }).then(() => {
        //update 'questions' table by adding the 'for_answer' column
        knex.schema.table('questions', function (table) {
            table.integer('for_answer')
                .references('mid').inTable('messages' /*'answers'*/)
                .onDelete('SET NULL'); //if answer is deleted, set for_answer to null
        });
        /*
        ^ as stated in https://www.postgresql.org/docs/10/static/ddl-inherit.html#DDL-INHERIT-CAVEATS,
        "A serious limitation of the inheritance feature is that indexes (including unique constraints) and foreign key constraints only apply to single tables, not to their inheritance children."
        it is not possible (with PostgresQL v10) to use both inheritance and foreign keys at once.
        until it is, or a workaround is found, we simply don't maintain child-level referential integrity
        and we hope all the rest will work, e.g. simple_group_gids will indeed refer to simple groups, and idem for meta_group_gids.
        TODO: maybe, check if PostgresQL's latest version enables this feature?
        */
    });
};

exports.down = function (knex, Promise) {
    return knex.schema.dropTable('answers');
};
